/*******************************************************************************
 * Quick access to the Components object.
 ******************************************************************************/
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
/*******************************************************************************
 * Component definitions.
 ******************************************************************************/
const SERVICE_CLASSNAME = "Netscape Sitemail Service";
const SERVICE_CLASSID = Components.ID("{759A7962-F7C2-479c-A18E-24292664852B}");
const SERVICE_CONTRACTID = "@netscape.com/sitemail/service;1";
/*******************************************************************************
 * Contract IDs.
 ******************************************************************************/
const PREF_CONTRACTID = "@mozilla.org/preferences-service;1";
const OB_CONTRACTID = "@mozilla.org/observer-service;1";
const REQUEST_CONTRACTID = "@mozilla.org/xmlextras/xmlhttprequest;1";
const TIMER_CONTRACTID = "@mozilla.org/timer;1";
const BUNDLE_CONTRACTID = "@mozilla.org/intl/stringbundle;1";
const ALERT_CONTRACTID = "@mozilla.org/alerts-service;1";
const MEDIATOR_CONTRACTID = "@mozilla.org/appshell/window-mediator;1";
const CACHE_CONTRACTID = "@netscape.com/cache/expiring;1";
const CONSOLE_CONTRACTID = "@mozilla.org/consoleservice;1";
const ERROR_CONTRACTID = "@mozilla.org/scripterror;1";
/*******************************************************************************
 * Observer topics
 ******************************************************************************/
const TOPIC_TIMER = "timer-callback";
const TOPIC_SITEMAIL = "netscape-sitemail";
const TOPIC_SHUTDOWN_APP = "quit-application";
const TOPIC_SHUTDOWN_XPCOM = "xpcom-shutdown";
const TOPIC_PREF = "nsPref:changed";
const TOPIC_ALERT = "alertclickcallback";
/*******************************************************************************
 * Default values used across components.
 ******************************************************************************/
const DEFAULT_BRANCH = "extensions.netscape.sitemail.";
const DEFAULT_BUNDLE = "chrome://browser/locale/netscape-sitemail/sitemail.properties";
const DEFAULT_SEC = 1000;                                    // 1 second
const DEFAULT_MIN = 60*DEFAULT_SEC;                          // 1 minute
const DEFAULT_HOUR = 60*DEFAULT_MIN;                         // 1 hour
const DEFAULT_DAY = 24*DEFAULT_HOUR;                         // 1 day
const DEFAULT_ERROR = "http://www.mozilla.org/newlayout/xml/parsererror.xml";
/*******************************************************************************
 * Status checker values.
 ******************************************************************************/
const CHECKER_URL = "endpoint.api";
const CHECKER_PAUSED = "Server error: " + SERVICE_CLASSNAME + " temporarily disabled for $1 minutes.";
/*******************************************************************************
 * Alert manager values.
 ******************************************************************************/
const ALM_TITLE = "sitemail.module.alert.title";
const ALM_TEXT = "sitemail.module.alert.text.";
const ALM_IMAGE = "chrome://browser/skin/netscape-sitemail/sitemail-alert.png";
const ALM_WIN = "navigator:browser";
const ALM_TIMER = 100;                                       // 100 milliseconds
/*******************************************************************************
 * Sign in checker values.
 ******************************************************************************/
const SIGNIN_PATTERNS = {
  homepage: "^https?:\/\/www\.netscape\.com/?$",
  messages: "^https?:\/\/www\.netscape\.com\/member/[^/]+/messages"
};
const SIGNIN_EXPIRES = DEFAULT_HOUR;                         // remove items after an hour
const SIGNIN_MAX = 100;                                      // hold a maximum of 100 items
const SIGNIN_TIMER = 100;                                    // 100 milliseconds
/*******************************************************************************
 * Sitemail service values.
 ******************************************************************************/
const SERVICE_SIGNIN = "http://www.netscape.com/signin/?backurl=messages";
const SERVICE_URL = "endpoint.page";
const SERVICE_UPDATE = "update";
const SERVICE_USIGNIN = "signin";
const SERVICE_USIGNOUT = "signout";
const SERVICE_DISABLED = "disabled";
const SERVICE_FREQ = "frequency";
/******************************************************************************
 * Get the main application window.  Use this if the type of window does matter.
 * 
 * @return  A window object used for modality.
 *****************************************************************************/
function getMainWindow()
{
  //get the app window
  var mediator = Cc[MEDIATOR_CONTRACTID].getService(Ci.nsIWindowMediator);
  var main = mediator.getMostRecentWindow("navigator:browser");
  if (main)
    return main;
                       
  //open a new window  
  var watcher = Cc[WATCHER_CONTRACTID].getService(Ci.nsIWindowWatcher);                 
  main = watcher.openWindow(null, "chrome://browser/content/browser.xul", 
                            "_blank", "chrome,all,dialog=no", "about:blank");  
  return main;
}
/*******************************************************************************
 * Function to log a message to the error console
 *
 * @param     message        Message to log
 ******************************************************************************/
function logMessage(message) {
  
  // create an error object
  var scriptError = Cc[ERROR_CONTRACTID].createInstance(Ci.nsIScriptError);
  var sourceName = __LOCATION__.path;
  var flag = Ci.nsIScriptError.warningFlag;
  scriptError.init(message, sourceName, "", 0, 0, flag, SERVICE_CONTRACTID);
  
  // log to console
  var consoleService = Cc[CONSOLE_CONTRACTID].getService(Ci.nsIConsoleService);
  consoleService.logMessage(scriptError);
}
/*******************************************************************************
 * Object used to check with the sitemail status api for sitemail data.
 *
 * @param     service        The sitemail service.
 ******************************************************************************/
function StatusChecker(service) {
  this.service = service;
  this.load();
}
StatusChecker.prototype = {

  _service: null,
  get service() { return this._service; },
  set service(val) { this._service = val; },
  
  _request: null,
  get request() { return this._request; },
  set request(val) { this._request = val; },
  
  _locked: null,
  get locked() { return this._locked; },
  set locked(val) { this._locked = val; },
    
  _errorCount: null,
  get errorCount() { return this._errorCount; },
  set errorCount(val) { this._errorCount = val; },
   
  _errorTimer: null,
  get errorTimer() { return this._errorTimer; },
  set errorTimer(val) { this._errorTimer = val; },
    
  // nsISupports
  QueryInterface: function sc_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },

  // nsIObserver
  observe: function sc_observe(subject, topic, data) {
  
    // do nothing if not timer topic
    if (topic != TOPIC_TIMER)
      return;
      
    // re-enable after timer
    this.service._paused = false;
  },
  
  load: function sc_load() {
    this.errorCount = 0;
    this.errorTimer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer); 
  },
  
  unload: function sc_unload() {
    
    // stop the timer
    this.errorTimer.cancel();
    
    // stop any requests
    if (this.request)
      this.request.abort();
    
    // remove variables
    this.service = null;
    this.request = null;  
    this.locked = null;
    this.errorCount = null;
    this.errorTimer = null;
  },
  
  requestUpdate: function sc_requestUpdate() {
    
    // request already pending
    if (this.locked)
      return;
             
    // create an XML HTTP request  
    var self = this;
    var request = Cc[REQUEST_CONTRACTID].createInstance(Ci.nsIXMLHttpRequest);
    var api = this.service._branch.getCharPref(CHECKER_URL); 
    request.open("GET", api, true);
    request.overrideMimeType("text/xml");
    request.setRequestHeader("X-Netscape", "Navigator");        
    request.QueryInterface(Ci.nsIJSXMLHttpRequest).onerror = function() { self.onError(); };
    request.QueryInterface(Ci.nsIJSXMLHttpRequest).onload = function() { self.onLoad(); };
    this.request = request;    
       
    // send the request
    this.locked = true;     
    this.request.send(null); 
  },
  
  onLoad: function sc_onLoad() {
      
    // get the vote item from the xml
    var request = this.request;
    var data = null;
    try {
      data = this.createData(request.responseXML);
    } catch(e) {}
        
    // data could not be created
    if (!data) {
      this.onError();
      return;
    }
    
    // release the request
    this.locked = false;
    this.request = null;

    // reset the error counter
    this.errorCount = 0;
        
    // cache the data and notify the front-end
    this.service._setData(data, true);
  },
  
  onError: function sc_onError() {
  
    // release the request 
    this.locked = false;
    this.request = null;
        
    // cache the data and notify the front-end
    var data = { userName: null, totalCount: 0 };
    this.service._setData(data, true);
    
    // log to the console
    this.errorCount++;
    var delay = Math.pow(this.errorCount, 2);
    var message = CHECKER_PAUSED.replace("$1", delay);
    logMessage(message);
        
    // increment the error counter and start the disabled timer   
    this.service._paused = true;
    this.errorTimer.init(this, delay*DEFAULT_MIN, Ci.nsITimer.TYPE_ONE_SHOT);    
  },
  
  createData: function sc_createData(xml) {
  
    // parse error
    if (!xml || xml.documentElement.namespaceURI == DEFAULT_ERROR)
      return null;
         
    // parse the sitemail data
    var data = {  userName: null, totalCount: 0};
    var member = xml.getElementsByTagName("member")[0];
    var signedIn = parseInt(member.getAttribute("loggedIn"));
    
    // not signed in so return early
    if (!signedIn)
      return data;

    // setup the data
    data.userName = member.getElementsByTagName("alias")[0].firstChild.nodeValue;    
    data.totalCount = parseInt(xml.getElementsByTagName("sitemail")[0].getElementsByTagName("messagecount")[0].firstChild.nodeValue);    
    return data;                         
  }  
};
/*******************************************************************************
 * Object used to display an alert notification.
 *
 * @param     service        The fas service.
 ******************************************************************************/
function AlertManager(service) {
  this.service = service;
  this.load();
}
AlertManager.prototype = {

  _timer: null,
  get timer() { return this._timer; },
  set timer(val) { this._timer = val; },

  _service: null,
  get service() { return this._service; },
  set service(val) { this._service = val; },
  
  _bundle: null,
  get bundle() { return this._bundle; },
  set bundle(val) { this._bundle = val; },

  _totalCount: null,
  get totalCount() { return this._totalCount; },
  set totalCount(val) { this._totalCount = val; },
  
  _prevCount: null,
  get prevCount() { return this._prevCount; },
  set prevCount(val) { this._prevCount = val; },
    
  // nsISupports
  QueryInterface: function alm_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
        
  // nsIObserver
  observe: function alm_observe(subject, topic, data) { 
    if (topic == TOPIC_ALERT)
      this.open(subject.QueryInterface(Ci.nsIPropertyBag));
    else if (topic == TOPIC_TIMER)
      this.fireTimer();
  },
   
  load: function alm_load() {
    // setup count
    this.totalCount = 0;
    this.prevCount = 0;
    
    // setup string bundle
    var sbService = Cc[BUNDLE_CONTRACTID].getService(Ci.nsIStringBundleService);
    this.bundle = sbService.createBundle(DEFAULT_BUNDLE); 
    
    // setup the timer
    this.timer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer);   
  },

  unload: function alm_unload() {

    // stop the timer
    this.timer.cancel();
    
    // remove the variables
    this.timer = null;
    this.service = null;
    this.bundle = null;
    this.totalCount = null;
    this.prevCount = null;    
  },
  
  fireTimer: function alm_fireTimer() {
    
    // no alerts
    if (this.totalCount < 1)
      return;
    
    // count has not changed
    if (this.totalCount == this.prevCount)
      return;
          
    // do nothing if we do not have the alert service
    var showAlert = (ALERT_CONTRACTID in Cc && "nsIAlertService" in Ci);
    if (!showAlert)
      return;
      
    // do nothing if not enabled
    showAlert = this.service._branch.getBoolPref("showAlert");
    if (!showAlert)
      return;
    
    // show the alert
    var service = Cc[ALERT_CONTRACTID].getService(Ci.nsIAlertsService);
    var title = this.bundle.GetStringFromName(ALM_TITLE);
    var text;
    if (this.totalCount == 1)
      text = ALM_TEXT + "single";   
    else
      text = ALM_TEXT + "plural";
    text = this.bundle.formatStringFromName(text, [this.totalCount], 1);
    service.showAlertNotification(ALM_IMAGE, title, text, true, null, this);
  },

  showAlert: function alm_showAlert(count) {
    this.prevCount = this.totalCount;  
    this.totalCount = count;
    this.timer.init(this, ALM_TIMER, Ci.nsITimer.TYPE_ONE_SHOT);
  },
  
  open: function alm_open(bag) {
    var win = getMainWindow();
    var e = {
      shiftKey: bag.getProperty("shiftKey"),
      ctrlKey: bag.getProperty("ctrlKey"),
      metaKey: bag.getProperty("metaKey"),
      altKey: bag.getProperty("altKey"),
      button: bag.getProperty("button")    
    };
    win.Sitemail.open(e);  
  }
};
/*******************************************************************************
 * Helper object to check for signin/signout activities.
 ******************************************************************************/
function SigninChecker(service) {
  this.service = service;
  this.load();
}
SigninChecker.prototype = {

  _service: null,
  get service() { return this._service; },
  set service(val) { this._service = val; },
  
  _cache: null,
  get cache() { return this._cache; },
  set cache(val) { this._cache = val; },
  
  _patterns: null,
  get patterns() { return this._patterns; },
  set patterns(val) { this._patterns = val; },
  
  _timer: null,
  get timer() { return this._timer; },
  set timer(val) { this._timer = val; },
  
  _isChecking: null,
  get isChecking() { return this._isChecking; },
  set isChecking(val) { this._isChecking = val; },
  
  _queue: null,
  get queue() { return this._queue; },
  set queue(val) { this._queue = val; },
          
  // nsISupports
  QueryInterface: function signin_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIObserver
  observe: function signin_observe(subject, topic, data) {
    if (topic != TOPIC_TIMER)
      return;
      
    this.fireTimer();
  },
            
  load: function signin_load() {

    // setup the variables
    this.cache = Cc[CACHE_CONTRACTID].createInstance(Ci.nsIExpiringCache);
    this.timer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer); 
    this.isChecking = false;          
    this.queue = [];
    var patterns = {};
    for (var pattern in SIGNIN_PATTERNS)
      patterns[pattern] = new RegExp(SIGNIN_PATTERNS[pattern], "i");
    this.patterns = patterns;
    
    // setup the cache
    this.cache.init(SIGNIN_MAX, SIGNIN_EXPIRES, true);
  },
  
  unload: function signin_unload() {
    
    // stop the timer
    this.timer.cancel();
    this.cache.destroy();
 
    // remove the variables
    this.cache = null;
    this.patterns = null;
    this.queue = null;
    this.service = null;
    this.isChecking = null;
  },

  fireTimer: function signin_fireTimer() {
    while (this.queue.length) {
      var URL = this.queue.shift();
      this.testURL(URL);
    }
    this.isChecking = false;
  },
  
  checkURL: function signin_checkURL(URL) {
      
    // already queued
    if (this.queue.indexOf(URL) != -1)
      return;
      
    // queue up the url
    this.queue[this.queue.length] = URL;
    
    // fire the timer
    if (!this.isChecking) {
      this.isChecking = true;
      this.timer.init(this, SIGNIN_TIMER, Ci.nsITimer.TYPE_ONE_SHOT);
    }
  },
  
  testURL: function signin_testURL(URL) {
    
    // get the value from the cache
    var value = this.cache.getValue(URL);
    
    // match patterns
    if (!value)
      value = this.matchURL(URL);
      
    // notify if not none  
    if (value != "none")
      this.service.checkActivity();
      
    // cache the value
    this.cache.setValue(URL, value);
  },
  
  matchURL: function signin_matchURL(URL) {
  
    // set the default value to none
    var rv = "none";
    
    // loop through the regexp patterns
    var patterns = this.patterns;
    for (var pattern in patterns) {
    
      // url matches pattern set rv and break
      var regex = patterns[pattern];
      if (URL.match(regex)) {
        rv = pattern;
        break;
      }
    }
    
    // return rv
    return rv;
  }   
};
/*******************************************************************************
 * The main sitemail interface.  This interface takes care of updating the 
 * mail list.  It notifies the front-end when updates take place.
 * 
 * @version   1.0
 ******************************************************************************/
function SitemailService() {
  this._load();
}
SitemailService.prototype = {
  _obService: null,
  _branch: null,
  _disabled: null,  
  _paused: null,
  _userName: null,
  _totalCount: null,
  _URL: null,
  _checker: null,
  _alerter: null,
  _timer: null,
  _signin: null,
        
  // nsISupports
  QueryInterface: function sms_QI(iid) {
    if (iid.equals(Ci.nsISitemailService) ||
        iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIObserver
  observe: function sms_observe(subject, topic, data) {
    switch (topic) {
    
    case TOPIC_SHUTDOWN_APP:
      this._unload();
      break;
              
    case TOPIC_SHUTDOWN_XPCOM:
      this._unloadFinal();
      break;
    
    case TOPIC_TIMER:
      this._tryCheck();
      break;
    
    case TOPIC_PREF:
      this._prefUpdate(data);
      break;
        
    default:
      return;
    }
  },
  
  get disabled() { return this._disabled; },
  get paused() { return this._paused; },  
  get isSignedIn() { return (this._userName !== null); },
  get userName() { return this._userName; },
  get hasMail() { return (this._totalCount > 0); },
  get totalCount() { return this._totalCount; },
  get URL() { return this._URL; },
  
  checkActivity: function sms_checkActivity() {
    
    // try a check  
    this._tryCheck();
          
    // stop the timer
    this._timer.cancel();
    
    // start the timer again
    var freq = DEFAULT_MIN * this._branch.getIntPref(SERVICE_FREQ);
    this._timer.init(this, freq, Ci.nsITimer.TYPE_REPEATING_SLACK);
  },
  
  checkURL: function sms_checkURL(URL) {
    this._signin.checkURL(URL);
  },
        
  _load: function sms__load() {
    
    // get the pref branch and observe it
    this._branch = Cc[PREF_CONTRACTID].getService(Ci.nsIPrefService).
                   getBranch(DEFAULT_BRANCH);
    this._branch.QueryInterface(Ci.nsIPrefBranch2).addObserver("", this, false);
    
    // get the observer service and add ourself as an observer               
    this._obService = Cc[OB_CONTRACTID].getService(Ci.nsIObserverService);
    this._obService.addObserver(this, TOPIC_SHUTDOWN_APP, false);
    this._obService.addObserver(this, TOPIC_SHUTDOWN_XPCOM, false);

    // setup the variables
    this._userName = null;
    this._totalCount = 0;
    this._URL = SERVICE_SIGNIN;  
    this._disabled = this._branch.getBoolPref(SERVICE_DISABLED);
    this._paused = false;
    this._checker = new StatusChecker(this);    
    this._alerter = new AlertManager(this);
    this._signin = new SigninChecker(this);
    
    // setup the timer
    this._timer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer);
    
    // start the update timer
    if (!this.disabled) { 
      var freq = DEFAULT_MIN * this._branch.getIntPref(SERVICE_FREQ);
      this._timer.init(this, freq, Ci.nsITimer.TYPE_REPEATING_SLACK);
      this._tryCheck();
    }
  },
  
  _unload: function sms__unload() {
    
    // remove the pref observer
    this._branch.QueryInterface(Ci.nsIPrefBranch2).removeObserver("", this);  
    
    // stop the update timer
    this._timer.cancel();
          
    // unload the checker and alerter
    this._checker.unload();
    this._alerter.unload(); 
    this._signin.unload(); 
  },
  
  _unloadFinal: function sms__unloadFinal() {
         
    // remove observers
    this._obService.removeObserver(this, TOPIC_SHUTDOWN_APP);    
    this._obService.removeObserver(this, TOPIC_SHUTDOWN_XPCOM);
     
    // remove variables
    this._obService = null;    
    this._branch = null;          
    this._disabled = null;
    this._paused = null;
    this._userName = null;
    this._totalCount = null; 
    this._URL = null;
    this._checker = null;
    this._alerter = null;
    this._timer = null; 
    this._signin = null;
  },
  
  _setData: function sms__setData(data, notify) {
  
     // determine sign
     var wasSignedIn = this.isSignedIn;
     var isSignedIn = true;
       
     // setup data
     this._userName = data.userName;
     this._totalCount = data.totalCount;
     if (this.isSignedIn) {
     
       // setup the url
       var url = this._branch.getCharPref(SERVICE_URL);
       url = url.replace("$1", this.userName);
       this._URL = url;
     
       // call the alerter
       this._alerter.showAlert(this.totalCount);
     } else {
       this._URL = SERVICE_SIGNIN;
       isSignedIn = false;
     }
     
     // determine notification
     var value = SERVICE_UPDATE;
     if (isSignedIn && !wasSignedIn)
       value = SERVICE_USIGNIN;
     else if (!isSignedIn && wasSignedIn)
       value = SERVICE_USIGNOUT;
                   
     // notify the front-end
     if (notify)
       this._notify(value);
  },
  
  _prefUpdate: function sms__prefUpdate(data) {
    var wasDisabled = this.disabled;
    var isDisabled = this._branch.getBoolPref(SERVICE_DISABLED);
    var freq = DEFAULT_MIN * this._branch.getIntPref(SERVICE_FREQ);
      
    // disabled changed
    if (data == SERVICE_DISABLED) {
      this._disabled = isDisabled;
      
      // Went from enabled to disabled stop the timer
      if (isDisabled && !wasDisabled) {
        this._timer.cancel();
        this._setData({userName: null, totalCount: 0}, false);
        this._notify(SERVICE_UPDATE);
        
      // Went from disabled to enabled start the timer
      } else if (!isDisabled && wasDisabled) {
        this._notify(SERVICE_UPDATE);      
        this._timer.init(this, freq, Ci.nsITimer.TYPE_REPEATING_SLACK);
        this._tryCheck();
      }
            
    // change the timer frequency
    } else if (!isDisabled) {
      this._timer.cancel();    
      this._timer.init(this, freq, Ci.nsITimer.TYPE_REPEATING_SLACK);
      this._tryCheck();
    }
  },
  
  _tryCheck: function sms__tryCheck() {
  
    // do nothing if disabled or paused
    if (this.disabled || this.paused)
      return;
      
    // request a check
    this._checker.requestUpdate();
  },
  
  _notify: function sms__notify(data) {
    try {
      this._obService.notifyObservers(null, TOPIC_SITEMAIL, data);
    } catch(e) {
      Components.utils.reportError(e);
    }
  }    
};
/*******************************************************************************
 * Generic object used as a factory based on a passed in constructor.
 ******************************************************************************/
function GenericFactory(ctor) {
  this._ctor = ctor;
}  
GenericFactory.prototype = {
  _ctor: null,
  
  // nsISupports
  QueryInterface: function gf_QI(iid) {
    if (iid.equals(Ci.nsIFactory) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIFactory
  createInstance: function gf_createInstance(outer, iid) {
    if (outer !== null)
      throw Cr.NS_ERROR_NO_AGGREGATION;
    return (new this._ctor()).QueryInterface(iid);
  },

  lockFactory: function gf_lockFactory(loc) {
    throw Cr.NS_ERROR_NOT_IMPLEMENTED;
  }
};
/*******************************************************************************
 * Object implementing the nsIModule interface.  This is used to retrieve 
 * the different components and services the module implements.
 ******************************************************************************/
var Module = {

  // nsISupports
  QueryInterface: function mod_QI(iid) {
    if (iid.equals(Ci.nsIModule) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE;
  },
  
  // nsIModule
  getClassObject: function mod_getClassObject(cm, cid, iid) {
    if (!iid.equals(Ci.nsIFactory))
      throw Cr.NS_ERROR_NOT_IMPLEMENTED;
    
    if (cid.equals(SERVICE_CLASSID))
      return new GenericFactory(SitemailService);
          
    throw Cr.NS_ERROR_NO_INTERFACE;
  },

  registerSelf: function mod_registerSelf(cm, file, location, type) {
    var cr = cm.QueryInterface(Ci.nsIComponentRegistrar);  
    cr.registerFactoryLocation(SERVICE_CLASSID, SERVICE_CLASSNAME, SERVICE_CONTRACTID,
                               file, location, type);                                                                                            
  },

  unregisterSelf: function mod_unregisterSelf(cm, location, type) {
    var cr = cm.QueryInterface(Ci.nsIComponentRegistrar);
    cr.unregisterFactoryLocation(SERVICE_CLASSID, location);
  },

  canUnload: function mod_canUnload(cm) { return true; }
};
/*******************************************************************************
 * Entry point used to retrieve the module object.
 ******************************************************************************/
function NSGetModule(cm, file) { return Module; }